package Services;

// Datei: WoerterbuchManager.java
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ServiceLoader;

public final class WoerterbuchManager
{
   private static WoerterbuchManager woerterbuchManager;

   private Map<String, IWoerterbuch> worterbuecherMap;

   // Singleton
   private WoerterbuchManager()
   {
      worterbuecherMap = new HashMap<String, IWoerterbuch>();
   }

   // Aktuelle Instanz des Singletons Woerterbuchmanager zurueckgeben
   public static WoerterbuchManager gibInstanz()
   {
      if (woerterbuchManager != null)
      {
         return woerterbuchManager;
      }
      else
      {
         woerterbuchManager = new WoerterbuchManager();
         return woerterbuchManager;
      }
   }

   // Laedt alle vorhandenen Woerterbuecher
   public void ladeVorhandeneWoerterbuecher()
   {
      ServiceLoader<IWoerterbuch> woerterbuecher = 
ServiceLoader.load(IWoerterbuch.class);
// Statischer Teil des Interfaces "IWoerterbuch" an
// ServicesLoader uebergeben, damit passende Plug-ins gefunden 
// werden
      for (IWoerterbuch aktuellesWoerterbuch : woerterbuecher)
      {
         worterbuecherMap.put(aktuellesWoerterbuch.gibSprache(),
               aktuellesWoerterbuch);
      }
   }

   // Uebersetzt das Wort in die gewuenschte Sprache
   public String gibUebersetzung(String sprache, String wort)
   {
      IWoerterbuch aktuellesWoerterbuch =
                                      worterbuecherMap.get(sprache);
      String uebersetztesWort = 
                         aktuellesWoerterbuch.gibUebersetzung(wort);

      return uebersetztesWort;
   }

   // Alle vorhandenen Woerterbuecher
   public List<String> gibAlleWoerterbuecher()
   {
      return new ArrayList<String>(worterbuecherMap.keySet());
   }
}
