// Datei: Broker.java
import java.util.HashMap;
public class Broker
{
   private HashMap<String, IClientsideProxy> clientMap;
   private HashMap<String, IServersideProxy> serverMap;

   public Broker()
   {
      clientMap = new HashMap<String, IClientsideProxy>();
      serverMap = new HashMap<String, IServersideProxy>();
      System.out.println ("Broker: instanziiert");
   }

   // Ein Server-side Proxy wird angemeldet. Anforderungen fuer den
   // angegebenen Dienst, koennen an ihn weitergeleitet werden.
   public void anmelden (String dienstName, IServersideProxy proxy)
   {
      serverMap.put (dienstName, proxy);
      System.out.println ("Broker: Dienst \"" + dienstName + 
              "\" angemeldet");
   }

   // Ein Dienst wird wieder abgemeldet und aus der 
   // serverMap entfernt.
   public void abmelden (String dienstName)
   {
      serverMap.remove (dienstName);
      System.out.println ("Broker: Dienst \"" + dienstName + 
              "\" abgemeldet");
   }

   // Diese Methode ruft ein Client-side Proxy auf,
   // um einen Dienst anzufragen. Der Dienst ist in der
   // Anfragenachricht enthalten ebenso wie der Clientname.
   public void anfrageWeiterleiten (byte[] anfrage, 
                                    IClientsideProxy proxy)
   {
      String clientName = ermittleClientName (anfrage);
      String dienstName = ermittleDienstName (anfrage);
      clientMap.put (clientName, proxy);
      IServersideProxy serverProxy = findeServer (dienstName);
      serverProxy.bearbeiteAnfrage (anfrage);
   }

   // Diese Methode ruft ein Server-side Proxy auf, der eine
   // Antwortnachricht als Ergebnis eines Dienstaufrufs
   // an den Client schicken will. Der Clientname ist in der
   // Antwortnachricht enthalten.
   public void antwortWeiterleiten (byte[] antwort)
   {
      String clientName = ermittleClientName (antwort);
      IClientsideProxy clientProxy = findeClient (clientName);
      clientProxy.bearbeiteAntwort (antwort);
      clientMap.remove (clientName);
   }

   // Diese Methode sucht in der serverMap nach dem Dienstnamen.
   private IServersideProxy findeServer (String dienst)
   {
      return serverMap.get (dienst);
   }

   // Diese Methode sucht in der clientMap nach dem Clientnamen.
   private IClientsideProxy findeClient (String client)
   {
      return clientMap.get (client);
   }

   // Ermittle Clientname aus Antwortnachricht
   private String ermittleClientName (byte[] nachricht)
   {
      String[] strings = (new String (nachricht)).split (";");
      return strings[0];
   }

   // Ermittelt Dienstname aus der Anfragenachricht.
   private String ermittleDienstName (byte[] anfrage)
   {
      String[] strings = (new String (anfrage)).split (";");
      return strings[1];
   }
}
